/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.task;

import java.awt.event.ActionEvent;
import java.util.Arrays;
import java.util.List;
import javax.swing.AbstractAction;
import javax.swing.Action;
import nl.cloudfarming.client.model.Entity;
import org.openide.awt.ActionID;
import org.openide.awt.ActionReference;
import org.openide.awt.ActionReferences;
import org.openide.awt.ActionRegistration;
import org.openide.util.*;

/**
 *
 * @author Maciek Dulkiewicz <maciek.dulkiewicz@ordina.nl>
 */
@ActionID(category = "Task", id = "nl.cloudfarming.client.task.core.AddEntityToTaskAction")
@ActionRegistration(iconBase = "nl/cloudfarming/client/task/shape/clipboard_sign.png", displayName = "Add to active task")
//@ActionReferences(value = {
//    @ActionReference(path = "Loaders/application/x-agrosense-cropfield/Actions", position = 2000),
//    @ActionReference(path = "Loaders/application/x-agrosense-field/Actions", position = 2000),
//    @ActionReference(path = "Loaders/application/x-agrosense-treatmentzone/Actions", position = 2000)
//})
@NbBundle.Messages({"AddEntityToTaskActionName=Add to active task"})
public class AddEntityToTaskAction extends AbstractAction implements LookupListener, ContextAwareAction {

    private Lookup context;
    private Lookup.Result<Entity> lkpInfo;
    private TaskService taskService = Lookup.getDefault().lookup(TaskService.class);

    public AddEntityToTaskAction() {
        this(Utilities.actionsGlobalContext());
    }

    public AddEntityToTaskAction(Lookup context) {
        putValue(Action.NAME, Bundle.AddEntityToTaskActionName());
        this.context = context;
    }

    void init() {
        //This line is commented because it causes trouble when running tests for this class
        //Please feel free to find a workaround and uncomment it once again!
//        assert SwingUtilities.isEventDispatchThread() : "this shall be called just from AWT thread";

        if (lkpInfo != null) {
            return;
        }

        //The thing we want to listen for the presence or absence of
        //on the global selection
        lkpInfo = context.lookupResult(Entity.class);
        lkpInfo.addLookupListener(this);
        resultChanged(null);
    }

    @Override
    public boolean isEnabled() {
        init();
        return super.isEnabled();
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        init();
        Task activeTask = taskService.getActiveTask();
        for (Entity entity : lkpInfo.allInstances()) {
            activeTask.getContext().addItem(entity);
        }
    }

    private boolean selectedEntitiesAreValid() {
        //Get all classes required by task
        List<Class> requiredClasses = Arrays.asList(taskService.getActiveTask().getRequiredClasses());

        //Check that the class of each selected entity is actually required
        for (Class clazz : lkpInfo.allClasses()) {
            //if a selected class is not required, we cant use it and the action is disabled
            if (!requiredClasses.contains(clazz)) {
                return false;
            }
        }

        //for each selected entity make sure it has a URI and is not already in the context of the task
        for (Entity entity : lkpInfo.allInstances()) {
            if (entity.getURI() == null || taskService.getActiveTask().getLookup().lookupAll(entity.getClass()).contains(entity)) {
                return false;
            }
        }

        return true;
    }

    @Override
    public void resultChanged(LookupEvent ev) {
        if (lkpInfo.allInstances().isEmpty() || taskService.getActiveTask() == null) {
            setEnabled(false);
        } else {
            setEnabled(selectedEntitiesAreValid());
        }
    }

    @Override
    public Action createContextAwareInstance(Lookup context) {
        return new AddEntityToTaskAction(context);
    }
}
