/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.task;

import java.awt.event.ActionEvent;
import java.util.HashMap;
import java.util.List;
import javax.swing.AbstractAction;
import org.openide.awt.ActionID;
import org.openide.awt.ActionRegistration;
import org.openide.util.Lookup;
import org.openide.util.NbBundle;

/**
 *
 * @author Johan Reitzema
 * @author Maciek Dulkiewicz
 */
@ActionID(category = "ExportableTask", id = "nl.cloudfarming.client.isobus.type.ExportAction")
@ActionRegistration(displayName = "#isobus.action.export")
@NbBundle.Messages({
    "isobus.action.export=Export Isobus",
    "isobus.action.progress_handler_msg=Exporting Isobus task data",
    "isobus.action.confirm_overwrite_taskdata=Overwrite current taskdata folder contents?"    
})
public class ExportAction extends AbstractAction {

    
    private final List<ExportableTaskCapability> exportCapabilities;


    public ExportAction(List<ExportableTaskCapability> exportCapabilities) {
        super(NbBundle.getMessage(ExportAction.class, "isobus.action.export"));
        this.exportCapabilities = exportCapabilities;
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        //TODO what to do with multiple implemenmtations?
        TaskExporter exporter = Lookup.getDefault().lookup(TaskExporter.class);
        ExportableTask[] tasks = new ExportableTask[exportCapabilities.size()];
        int i=0;
        for (ExportableTaskCapability capability: exportCapabilities){
            tasks[i++]=capability.getTask();
        }
        exporter.exportTask(new HashMap<String,Object>(),tasks);

    }
}
