/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.task.factory;

import java.awt.Dimension;
import java.util.List;
import javax.swing.event.ChangeListener;
import nl.cloudfarming.client.util.swing.ASBodyPanel;
import org.openide.WizardDescriptor;
import org.openide.util.HelpCtx;

/**
 * Controller for the TaskObjectFactory selection wizard panel. 
 * 
 * @author Gerben Feenstra
 */
public class TaskObjectFactorySelectController implements WizardDescriptor.Panel<WizardDescriptor> {

    public static final String PROP_SELECTED_TASKOBJECTFACTORY = "SelectedFactory";
    
    private TaskObjectFactorySelectPanel visualPanel;
    private ASBodyPanel component;
    private List<TaskObjectFactory> supportedFactories;
    private String panelTitle;

    public TaskObjectFactorySelectController(List<TaskObjectFactory> supportedFactories, String panelTitle, String panelText, Dimension panelSize) {
        this.supportedFactories = supportedFactories;
        this.panelTitle = panelTitle;
        visualPanel = new TaskObjectFactorySelectPanel(panelText);
        visualPanel.setMinimumSize(panelSize);
        visualPanel.setPreferredSize(panelSize);
    }

    // Get the visual component for the panel. In this template, the component
    // is kept separate. This can be more efficient: if the wizard is created
    // but never displayed, or not all panels are displayed, it is better to
    // create only those which really need to be visible.
    @Override
    public ASBodyPanel getComponent() {
        if (component == null) {
            visualPanel.setTaskObjectFactories(supportedFactories);
            component = new ASBodyPanel(visualPanel);
            component.setName(panelTitle);
            component.setTitle(panelTitle);
        }
        return component;
    }

    @Override
    public HelpCtx getHelp() {
        return null;
    }

    /**
     * Used to retrieve previous panel state. Not implemented for this panel.
     *
     * @param wiz
     */
    @Override
    public void readSettings(WizardDescriptor wiz) {
    }

    /**
     * Store selected Geometricals and selected project in the wizardDescriptor
     * for retrievel after this wizard finishes.
     *
     * @param wiz
     */
    @Override
    public void storeSettings(WizardDescriptor wiz) {
        wiz.putProperty(PROP_SELECTED_TASKOBJECTFACTORY, visualPanel.getComboBoxModel().getSelectedItem());
    }

    @Override
    public boolean isValid() {
        return true;
    }

    @Override
    public void addChangeListener(ChangeListener l) {
    }

    @Override
    public void removeChangeListener(ChangeListener l) {
    }
}
