/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.task.factory;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.List;
import javax.swing.JComponent;
import org.openide.DialogDisplayer;
import org.openide.WizardDescriptor;
import org.openide.util.NbBundle.Messages;

/**
 * TaskObjectFactory selection wizard. 
 * Can be used when more then one TaskObjectFactory is available in the lookup. 
 * Will present the user with a selection wizard to select which factory to use.
 * 
 * @author Gerben Feenstra
 */
@Messages({
    "TaskObjectFactorySelectWizard_step_selectiontype_title=Select treatmentzone source",
})
public final class TaskObjectFactorySelectWizard implements ActionListener {

    private static final String SELECTION_TYPE_TITLE = Bundle.TaskObjectFactorySelectWizard_step_selectiontype_title();
    private TaskObjectFactory selectedTaskObjectfactory = null;
    private List<TaskObjectFactory> supportedFactories;
    private String panelTitle;
    private String panelText;
    private Dimension panelSize;
    
    public TaskObjectFactorySelectWizard(List<TaskObjectFactory> supportedFactories, String title, String panelText, Dimension panelSize){
        this.supportedFactories = supportedFactories;
        this.panelTitle = title;
        this.panelText = panelText;
        this.panelSize = panelSize; 
    }
    
    @Override
    public void actionPerformed(ActionEvent e) {
        List<WizardDescriptor.Panel<WizardDescriptor>> panels = new ArrayList<WizardDescriptor.Panel<WizardDescriptor>>();
        panels.add(new TaskObjectFactorySelectController(supportedFactories, panelTitle, panelText, panelSize));
        
        String[] steps = new String[panels.size()];
        for (int i = 0; i < panels.size(); i++) {
            Component c = panels.get(i).getComponent();
            // Default step name to component name of panel.
            steps[i] = c.getName();
            if (c instanceof JComponent) { // assume Swing components
                JComponent jc = (JComponent) c;
                jc.putClientProperty(WizardDescriptor.PROP_CONTENT_SELECTED_INDEX, i);
                jc.putClientProperty(WizardDescriptor.PROP_CONTENT_DATA, steps);
                jc.putClientProperty(WizardDescriptor.PROP_AUTO_WIZARD_STYLE, true);
                jc.putClientProperty(WizardDescriptor.PROP_CONTENT_DISPLAYED, true);
                jc.putClientProperty(WizardDescriptor.PROP_CONTENT_NUMBERED, false);
            }
        }
        WizardDescriptor wiz = new WizardDescriptor(new WizardDescriptor.ArrayIterator<WizardDescriptor>(panels));
        // {0} will be replaced by WizardDesriptor.Panel.getComponent().getName()
        wiz.setTitleFormat(new MessageFormat("{0}"));
        wiz.setTitle(SELECTION_TYPE_TITLE);
        if (DialogDisplayer.getDefault().notify(wiz) == WizardDescriptor.FINISH_OPTION) {
            selectedTaskObjectfactory = (TaskObjectFactory) wiz.getProperty(TaskObjectFactorySelectController.PROP_SELECTED_TASKOBJECTFACTORY);
        }
    }
    
    /**
     * Used to ask the wizard the selected factory when the wizard is completed.
     * @return 
     */
    public TaskObjectFactory getSelectedFactory(){
        return selectedTaskObjectfactory;
    }
}
