/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.task.windows;

import java.awt.BorderLayout;
import java.awt.Color;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import javax.swing.*;
import nl.cloudfarming.client.geoviewer.Palette;
import nl.cloudfarming.client.geoviewer.render.GridCoverage2DPreviewPanel;
import nl.cloudfarming.client.task.ExportableTask;
import nl.cloudfarming.client.util.swing.BodyPanel;
import nl.cloudfarming.client.util.swing.MessagePanel;
import org.netbeans.core.spi.multiview.CloseOperationState;
import org.netbeans.core.spi.multiview.MultiViewElement;
import org.netbeans.core.spi.multiview.MultiViewElementCallback;
import org.openide.awt.UndoRedo;
import org.openide.util.Lookup;
import org.openide.util.NbBundle.Messages;
import org.openide.windows.TopComponent;

/**
 *
 * @author Maciek Dulkiewicz
 */
@MultiViewElement.Registration(displayName = "#ExportableTaskPreviewElement_display_name",
iconBase = "nl/cloudfarming/client/task/to_do_list_checked_1_16.png",
mimeType = ExportableTask.MIME_TYPE,
persistenceType = TopComponent.PERSISTENCE_NEVER,
preferredID = "ExportableTaskPreview",
position = 200)
@Messages({
    "ExportableTaskPreviewElement_display_name=Grid preview",
    "ExportableTaskPreviewElement_loading_panel_label=Loading grid",
    "ExportableTaskPreviewElement_warn_no_values=Nothing to display, none of the treatment zones have values",
})
public final class ExportableTaskPreviewElement extends TopComponent implements MultiViewElement {

    private JToolBar toolbar = new JToolBar();
    private transient MultiViewElementCallback callback;
    private Lookup lookup;
    private ExportableTask task;
    private ExportableTask.ExportableTaskGeographical geographical;

    private BodyPanel bodyPanel; // decorator
    private JPanel contentPanel;
    private MessagePanel messagePanel;
    private GridCoverage2DPreviewPanel gridPanel;
    
    private final TaskListener taskListener = new TaskListener();
    
    private static final Color BACKGROUND_COLOR = Color.WHITE;

    public ExportableTaskPreviewElement(Lookup lookup) {
//        initComponents();
        this.lookup = lookup;
        this.task = lookup.lookup(ExportableTask.class);
        
        setName(Bundle.ExportableTaskPreviewElement_display_name());
        init();
    }
    
    private void init() {
        contentPanel = new JPanel();
        contentPanel.setLayout(new BorderLayout());
        
        messagePanel = new MessagePanel(MessagePanel.LARGE_ICON);
        updateMessages();
        
        gridPanel = new GridCoverage2DPreviewPanel(
                lookup.lookup(ExportableTask.ExportableTaskGeographical.class),
                lookup.lookup(Palette.class));
        
        contentPanel.add(messagePanel, BorderLayout.NORTH);
        contentPanel.add(gridPanel, BorderLayout.CENTER);
        
        bodyPanel = BodyPanel.create(contentPanel);
    }

    @Override
    public JComponent getVisualRepresentation() {
        return bodyPanel;
    }

    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 400, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 300, Short.MAX_VALUE)
        );
    }// </editor-fold>//GEN-END:initComponents

    // Variables declaration - do not modify//GEN-BEGIN:variables
    // End of variables declaration//GEN-END:variables
    /**
     * This method is called as soon as the ExportableTaskPreviewElement has
     * been opened and the panel has been filled with the default loadingPanel.
     * This method makes sure the initial version of the grid preview is
     * generated by the task.
     */
    @Override
    public void componentOpened() {
        task.addPropertyChangeListener(taskListener);
    }

    @Override
    public void componentClosed() {
        task.removePropertyChangeListener(taskListener);
    }

    void writeProperties(java.util.Properties p) {
        // better to version settings since initial version as advocated at
        // http://wiki.apidesign.org/wiki/PropertyFiles
        p.setProperty("version", "1.0");
        // TODO store your settings
    }

    void readProperties(java.util.Properties p) {
        String version = p.getProperty("version");
    }

    @Override
    public JComponent getToolbarRepresentation() {
        return toolbar;
    }

    @Override
    public Action[] getActions() {
        return new Action[0];
    }

    @Override
    public Lookup getLookup() {
        return lookup;
    }

    @Override
    public void componentShowing() {
    }

    @Override
    public void componentHidden() {
    }

    @Override
    public void componentActivated() {
//        initGridPreview();
    }

    @Override
    public void componentDeactivated() {
    }

    @Override
    public UndoRedo getUndoRedo() {
        return UndoRedo.NONE;
    }

    @Override
    public void setMultiViewCallback(MultiViewElementCallback callback) {
        this.callback = callback;
    }

    @Override
    public CloseOperationState canCloseElement() {
        return CloseOperationState.STATE_OK;
    }

    @Override
    public int getPersistenceType() {
        return PERSISTENCE_NEVER;
    }
    
    private void updateMessages() {
        // current validation is too strict for showing previews.
//        for (ExportableTaskError e : task.validate()) {...}
        
        if (!task.hasTreatmentZoneValues()) {
            messagePanel.setWarning(Bundle.ExportableTaskPreviewElement_warn_no_values());
        } else {
            messagePanel.setValid();
        }
    }
    
    private class TaskListener implements PropertyChangeListener {

        @Override
        public void propertyChange(PropertyChangeEvent evt) {
            updateMessages();
        }
    }
}
